@@ -29,6 +29,7 @@
 #include <QVBoxLayout>
 #include <QHBoxLayout>
 #include <QLabel>
+#include <QCheckBox>
 #include <QToolBar>
 #include <QtWebEngineWidgets/QWebEngineView>
 #include <QtWebEngineWidgets/QWebEngineSettings>
@@ -62,7 +63,7 @@
 #include "MainApplication.h"
 #include "MainWindow.h"
 
-#define DBG if(1)
+#define DBG if(0)
 
 static const QString SETTINGS_GROUP = "mainwindow";
 
@@ -116,6 +117,9 @@ MainWindow::MainWindow(QString filepath, QWidget *parent)
     m_MediaList(QStringList()),
     m_MediaKind(QStringList()),
     m_MediaBase(QString()),
+    m_skipPrintWarnings(false),
+    m_skipPrintPreview(false),
+    m_WebViewPrinter(new WebViewPrinter(this)),
     m_LastPtr(-1)
 {
     ui.setupUi(this);
@@ -545,6 +549,7 @@ void MainWindow::SetupView()
 
     ui.actionOpen->setEnabled(true);
     ui.actionSave->setEnabled(true);
+    ui.actionPrint->setEnabled(true);
     ui.actionExit->setEnabled(true);
     
     ui.actionUndo->setEnabled(true);
@@ -1101,6 +1106,8 @@ void MainWindow::moveEvent(QMoveEvent *event)
 void MainWindow::LoadSettings()
 {
     SettingsStore settings;
+    m_skipPrintWarnings = settings.skipPrintWarnings();
+    m_skipPrintPreview = settings.skipPrintPreview();
     settings.beginGroup(SETTINGS_GROUP);
     // The size of the window and its full screen status
     // Due to the 4.8 bug, we restore its "normal" window size and then maximize
@@ -1193,6 +1200,8 @@ void MainWindow::LoadSettings()
 void MainWindow::SaveSettings()
 {
     SettingsStore settings;
+    settings.setSkipPrintWarnings(m_skipPrintWarnings);
+    settings.setSkipPrintPreview(m_skipPrintPreview);
     settings.beginGroup(SETTINGS_GROUP);
     // The size of the window and it's full screen status                                                       
     // This is a workaround for this bug:                                                                       
@@ -1549,6 +1558,55 @@ bool MainWindow::Save()
     return save_result;
 }
 
+void MainWindow::printRendered()
+{
+#if QT_VERSION >= QT_VERSION_CHECK(5, 12, 0)
+    // Refresh skipflags from Prefs
+    SettingsStore settings;
+    m_skipPrintWarnings = settings.skipPrintWarnings();
+    m_skipPrintPreview = settings.skipPrintPreview();
+
+    if (!m_skipPrintWarnings) {
+        QCheckBox *cb = new QCheckBox(tr("Do not show this warning again"), this);
+        QString text = tr("This file may not print the way you expect it to.");
+        QString detailed_text = tr("Dark backgrounds and colored text applied with an EPUB's CSS will print.");
+        detailed_text = detailed_text + " " + tr("Use caution as this can result in a lot of ink being used!");
+        detailed_text = detailed_text + " " + tr("Use the following Print Preview to see how this file will print.");
+        detailed_text = detailed_text + " " + tr("Check the box if you don't wish to see this warning in the future.");
+        QMessageBox msgbox;
+        msgbox.setWindowFlags(Qt::Window | Qt::WindowStaysOnTopHint);
+        msgbox.setModal(true);
+        msgbox.setWindowTitle("PageEdit");
+        msgbox.setText("<h3>" + text + "</h3><br/>");
+        msgbox.setIcon(QMessageBox::Icon::Warning);
+        msgbox.setTextFormat(Qt::RichText);
+        msgbox.setDetailedText(detailed_text);
+        msgbox.setStandardButtons(QMessageBox::Close);
+        msgbox.setCheckBox(cb);
+        connect(cb, &QCheckBox::stateChanged, [this](int state) {
+            if (static_cast<Qt::CheckState>(state) == Qt::CheckState::Checked) {
+                m_skipPrintWarnings = true;    
+            }
+        });
+        msgbox.exec();
+    }
+    settings.setSkipPrintWarnings(m_skipPrintWarnings);
+
+    m_WebViewPrinter->setPage(m_WebView->url(), m_skipPrintPreview);
+#else
+    QMessageBox msgbox;
+    QString text = tr("Feature not available before Qt5.12.x");
+    msgbox.setWindowFlags(Qt::Window | Qt::WindowStaysOnTopHint);
+    msgbox.setModal(true);
+    msgbox.setWindowTitle("PageEdit");
+    msgbox.setText("<h3>" + text + "</h3><br/>");
+    msgbox.setIcon(QMessageBox::Icon::Warning);
+    msgbox.setStandardButtons(QMessageBox::Close);
+    msgbox.exec();
+#endif
+
+}
+
 void MainWindow::Open()
 {
 
@@ -1977,6 +2035,7 @@ void MainWindow::ConnectSignalsToSlots()
     // File Related
     connect(ui.actionOpen,         SIGNAL(triggered()), this, SLOT(Open()));
     connect(ui.actionSave,         SIGNAL(triggered()), this, SLOT(Save()));
+    connect(ui.actionPrint,        SIGNAL(triggered()), this, SLOT(printRendered()));
     connect(ui.actionSaveAs,       SIGNAL(triggered()), this, SLOT(SaveAs()));
     connect(ui.actionExit,         SIGNAL(triggered()), this, SLOT(Exit()));
     connect(ui.actionPreferences,  SIGNAL(triggered()), this, SLOT(PreferencesDialog()));
